package org.nakedobjects.nof.reflect.remote;

import java.util.Enumeration;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedReference;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.peer.ActionPeer;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nof.reflect.remote.data.DummyNullValue;
import org.nakedobjects.nof.reflect.remote.data.DummyReferenceData;
import org.nakedobjects.nof.reflect.remote.data.DummyResultData;
import org.nakedobjects.nof.reflect.remote.data.KnownObjects;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.nof.reflect.remote.data.ServerActionResultData;
import org.nakedobjects.nof.testsystem.TestPojo;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;
import org.nakedobjects.nof.testsystem.TestProxySystem;

import test.org.nakedobjects.object.reflect.DummyIdentifier;

import static org.easymock.EasyMock.aryEq;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.eq;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;


public class ProxyAction_ExecuteRemotelyTest extends TestCase {

    private ProxyAction proxy;
    private ActionPeer actionPeer;
    private ObjectEncoder encoder;
    private Distribution remoteInterface;
    private NakedObject target;
    private NakedReference param1;
    private TestProxySystem system;
    private DummyIdentifier identifier;
    private ReferenceData targetData;
    private Data[] parameterData;
    private Naked[] parameters;
    private String identifierString;

    protected void setUp() throws Exception {
        BasicConfigurator.configure();
        LogManager.getRootLogger().setLevel(Level.OFF);

        system = new TestProxySystem();
        system.init();

        actionPeer = createMock(ActionPeer.class);
        encoder = createMock(ObjectEncoder.class);
        remoteInterface = createMock(Distribution.class);

        proxy = new ProxyAction(actionPeer, remoteInterface, encoder);

        actionPeer.getIdentifier();
        identifier = new DummyIdentifier();
        expectLastCall().andStubReturn(identifier);
        identifierString = identifier.getClassName() + "#" + identifier.getName();
        target = system.createTransientTestObject();
        parameters = new Naked[] { param1, param1 };
        NakedObjectSpecification[] parameterTypes = new NakedObjectSpecification[] { system.getSpecification(TestPojo.class),
                system.getSpecification(TestPojo.class) };

        actionPeer.getParameterTypes();
        expectLastCall().andStubReturn(parameterTypes);

        actionPeer.getType();
        expectLastCall().andReturn(NakedObjectAction.USER);

        KnownObjects encodersKnownObjects = new KnownObjects();

        targetData = new DummyReferenceData();
        parameterData = new Data[] { null, null };
        encoder.createParameters(parameterTypes, parameters, encodersKnownObjects);
        expectLastCall().andReturn(parameterData);

        encoder.createActionTarget(target, encodersKnownObjects);
        expectLastCall().andReturn(targetData);

        encoder.madePersistent(null, null);
        expectLastCall().times(2);

    }

    public void testOnTransientObjectWithRemoteAnnotation() throws Exception {
        actionPeer.getTarget();
        expectLastCall().andStubReturn(NakedObjectAction.REMOTE);

        remoteInterface.executeServerAction(NakedObjectsContext.getSession(), NakedObjectAction.USER.getName(), identifierString,
                targetData, parameterData);
        ServerActionResultData result = new DummyResultData(new DummyNullValue("type"), new ObjectData[0], new ReferenceData[0], null,
                new ObjectData[2], new String[0], new String[0]);
        expectLastCall().andReturn(result);

        encoder.madePersistent(target, null);
        expectLastCall();

        replay(actionPeer, encoder, remoteInterface);
        proxy.execute(target, parameters);
        verify(actionPeer, encoder, remoteInterface);
    }

    public void testOnPersistent() throws Exception {
        actionPeer.getTarget();
        expectLastCall().andStubReturn(null);

        NakedObjectsContext.getObjectPersistor().makePersistent(target);

        remoteInterface.executeServerAction(NakedObjectsContext.getSession(), NakedObjectAction.USER.getName(), identifierString,
                targetData, parameterData);
        ServerActionResultData result = new DummyResultData(new DummyNullValue("type"), new ObjectData[0], new ReferenceData[0], null,
                new ObjectData[2], new String[0], new String[0]);
        expectLastCall().andReturn(result);

        replay(actionPeer, encoder, remoteInterface);
        proxy.execute(target, parameters);
        verify(actionPeer, encoder, remoteInterface);
    }

    public void testObjectsDestroyed() throws Exception {
        actionPeer.getTarget();
        expectLastCall().andStubReturn(null);

        NakedObjectsContext.getObjectPersistor().makePersistent(target);

        TestProxyNakedObject object = system.createPersistentTestObject();
        
        remoteInterface.executeServerAction(NakedObjectsContext.getSession(), NakedObjectAction.USER.getName(), identifierString,
                targetData, parameterData);
        ReferenceData[] disposedReferenceData = new ReferenceData[] { new DummyReferenceData(object.getOid(), object.getSpecification().getFullName(), null) };
        ServerActionResultData result = new DummyResultData(new DummyNullValue("type"), new ObjectData[0], disposedReferenceData, null,
                new ObjectData[2], new String[0], new String[0]);
        expectLastCall().andReturn(result);

        replay(actionPeer, encoder, remoteInterface);
        proxy.execute(target, parameters);
        verify(actionPeer, encoder, remoteInterface);
        
        Enumeration allDisposedObjects = NakedObjectsContext.getUpdateNotifer().allDisposedObjects();
        assertEquals(true, allDisposedObjects.hasMoreElements());
        assertEquals(object, allDisposedObjects.nextElement());
        assertEquals(false, allDisposedObjects.hasMoreElements());
    }
}

// Copyright (c) Naked Objects Group Ltd.
