package org.nakedobjects.nof.reflect.remote;

import java.util.Enumeration;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.easymock.EasyMock;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.persist.AllInstances;
import org.nakedobjects.nof.core.persist.SimpleOidGenerator;
import org.nakedobjects.nof.core.persist.TransactionException;
import org.nakedobjects.nof.reflect.remote.data.ClientTransactionEvent;
import org.nakedobjects.nof.reflect.remote.data.CriteriaData;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nof.reflect.remote.data.DummyClientResultData;
import org.nakedobjects.nof.reflect.remote.data.DummyIdentityData;
import org.nakedobjects.nof.reflect.remote.data.DummyNullValue;
import org.nakedobjects.nof.reflect.remote.data.DummyObjectData;
import org.nakedobjects.nof.reflect.remote.data.DummyReferenceData;
import org.nakedobjects.nof.reflect.remote.data.DummyValueData;
import org.nakedobjects.nof.reflect.remote.data.IdentityData;
import org.nakedobjects.nof.reflect.remote.data.KnownObjects;
import org.nakedobjects.nof.reflect.remote.data.NullData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.nof.testsystem.TestProxyOid;
import org.nakedobjects.nof.testsystem.TestProxySystem;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.DummyVersion;

import test.org.nakedobjects.object.persistence.defaults.DummyOidGenerator;
import test.org.nakedobjects.objects.bom.Movie;
import test.org.nakedobjects.objects.bom.Person;
import test.org.nakedobjects.objects.specification.MovieSpecification;

import static org.easymock.EasyMock.*;


public class ProxyPersistorTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(ProxyPersistorTest.class);
    }
 
    private Distribution remoteInterface;
    private ProxyPersistor proxy;
    private Session session;
    private TestProxySystem system;
    private ObjectEncoder encoder;

    protected void setUp() throws Exception {
        BasicConfigurator.configure();
        LogManager.getRootLogger().setLevel(Level.OFF);

        proxy = new ProxyPersistor();

        remoteInterface = createMock(Distribution.class);

        proxy.setConnection(remoteInterface);

        encoder = createMock(ObjectEncoder.class);
        proxy.setEncoder(encoder);

        proxy.setOidGenerator(new DummyOidGenerator());

        system = new TestProxySystem();
   //     system.setObjectPersistor(proxy);
        system.init();
        
        reset(remoteInterface);
       // remoteInterfaceControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
        
        session = NakedObjectsContext.getSession();
    }
    
    protected void tearDown() throws Exception {
    }
    
    public void testMakePersistentOutsideTransaction() throws Exception {
        system.setObjectPersistor(proxy);

        NakedObject transientObject = new DummyNakedObject();
        try {
            proxy.makePersistent(transientObject);
            fail();
        } catch (TransactionException e) {}
    }

    public void testClientSideMakesNoRemoteCallsWhenNoWorkNeeded() throws Exception {
        system.setObjectPersistor(proxy);
        replay();

        proxy.startTransaction();
        proxy.endTransaction();

        verify();
    }
    public void testFindInstances() throws Exception {
        system.setObjectPersistor(proxy);

        //system.addSpecificationToLoader(new MovieSpecification());

        // The remote interface is asked for instances, which are returned as data objects
        DummyObjectData instanceData = new DummyObjectData(new DummyOid(12, true), Movie.class.getName(), true, new DummyVersion(3));
        
        // The data then needs to be decoded into the naked objects
        DummyNakedObject dummyNakedObject = new DummyNakedObject(new DummyOid(12, true), ResolveState.GHOST, "test");
        dummyNakedObject.setupObject(new Movie());
        dummyNakedObject.setupSpecification(new MovieSpecification());

        reset(encoder);
        expect(encoder.restore(instanceData)).andReturn(dummyNakedObject);

        CriteriaData c = new TestCriteria();
        remoteInterface.findInstances(session, c);
        expectLastCall().andReturn(new ObjectData[] { instanceData });
        
        // run test
        AllInstances criteria = new AllInstances(NakedObjectsContext.getReflector().loadSpecification(Movie.class), false);
        encoder.createCriteriaData(criteria);
        expectLastCall().andReturn(c);
        
        
        replay();
        NakedCollection instances = proxy.findInstances(criteria);
        verify();

        // the proxy should return one instance, which will be the dummy object created by the encoder's restore call
        assertEquals(1, instances.size());
        assertEquals(dummyNakedObject, instances.firstElement());
    }

    private void verify() {
        EasyMock.verify(encoder, remoteInterface);
    }

    private void replay() {
        EasyMock.replay(encoder, remoteInterface);
    }

    public void testResolveImmediatelyIgnoredWhenAlreadyResolving() throws Exception {
        system.setObjectPersistor(proxy);

        NakedObject object = new DummyNakedObject(null, ResolveState.RESOLVING);
        
        // implicit: expect no calls to encoder object

        replay();
        proxy.resolveImmediately(object);
        verify();
    }
    
    
    public void testResolveImmediately() throws Exception {
        system.setObjectPersistor(proxy);

  /*      DummyObjectData instance1 = new DummyObjectData(new DummyOid(5), Movie.class.getName(), true, new DummyVersion(3));
        DummyValueData name = new DummyValueData("ET", "java.lang.String");
        instance1.setFieldContent(new Data[] { new DummyNullValue(Person.class.getName()), name });
*/
   
        /*DummyReferenceData ref = new DummyReferenceData(new DummyOid(5), Movie.class.getName(), null);
        remoteInterface.resolveImmediately(session, ref);
        remoteInterfaceControl.setReturnValue(instance1);

        remoteInterfaceControl.replay();

        NakedObject object;
        object = loader.recreateAdapterForPersistent(new DummyOid(5), NakedObjectsContext.getSpecificationLoader().loadSpecification(
                Movie.class));
        Movie movie = (Movie) object.getObject();
        assertEquals(null, movie.getName());
*/
        
        
        NakedObject object = new DummyNakedObject(new DummyOid(5), ResolveState.GHOST);
        
        // encoder used to create identitiy data for target object
        IdentityData identityData = new DummyReferenceData();
        expect(encoder.createIdentityData(object)).andReturn(identityData);
        
        // remote call asks for object's data
        ObjectData objectData = new DummyObjectData();
        expect(remoteInterface.resolveImmediately(session, identityData)).andReturn(objectData);
        
        // data decode expected
        expect(encoder.restore(objectData)).andReturn(null);
        
        
        replay();
        proxy.resolveImmediately(object);
        verify();
        
        /*
        
        assertEquals("ET", movie.getName());
        assertEquals(new DummyOid(5), object.getOid());
        assertEquals(new DummyVersion(3), object.getVersion());
        */
    }

    public void testResolveField_TBI() {}
    
    public void testHasInstances() throws Exception {
        system.setObjectPersistor(proxy);

        remoteInterface.hasInstances(session, Movie.class.getName());
        expectLastCall().andReturn(true);
        expectLastCall().andReturn(false);

        EasyMock.replay(remoteInterface);

        NakedObjectSpecification type = NakedObjectsContext.getReflector().loadSpecification(Movie.class);
        assertTrue(proxy.hasInstances(type, false));
        assertFalse(proxy.hasInstances(type, false));

        EasyMock.verify(remoteInterface);
    }

    public void testFindInstancesButNoneFound() throws Exception {
        system.setObjectPersistor(proxy);

        //system.addSpecificationToLoader(new MovieSpecification());
        

        CriteriaData c = new TestCriteria();
        remoteInterface.findInstances(session, c);
        expectLastCall().andReturn(new ObjectData[0]);
        
        AllInstances criteria = new AllInstances(NakedObjectsContext.getReflector().loadSpecification(Movie.class), false);
        expect(encoder.createCriteriaData(criteria)).andReturn(c);

       EasyMock.replay(remoteInterface);
       EasyMock.replay(encoder);

        proxy.findInstances(criteria);

        EasyMock.verify(remoteInterface);
        EasyMock.verify(encoder);
    }


    public void testClientSideActionThrowsExceptionWhenTransactionNotStarted() throws Exception {
        system.setObjectPersistor(proxy);

        try {
            proxy.endTransaction();
            fail();
        } catch (TransactionException expected) {
        }
    }
    

    public void testClientSideActionWhereObjectDeleted() throws Exception {
        NakedObject movieAdapter = system.createPersistentTestObject();
        system.setObjectPersistor(proxy);

        DummyIdentityData identityOfObjectToDelete = new DummyIdentityData();
        expect(encoder.createIdentityData(movieAdapter)).andReturn(identityOfObjectToDelete);
        
        remoteInterface.executeClientAction( EasyMock.eq(session), EasyMock.aryEq(new ReferenceData[] {identityOfObjectToDelete}), EasyMock.aryEq(new int[] {ClientTransactionEvent.DELETE}));
        Version[] versionUpdates = new Version[] {};
        expectLastCall().andReturn(new DummyClientResultData(new ObjectData[] {}, versionUpdates, new ObjectData[0]));
        
        replay();
        
        proxy.startTransaction();
        proxy.destroyObject(movieAdapter);
        proxy.endTransaction();

        verify();
        
        Enumeration allDisposedObjects = NakedObjectsContext.getUpdateNotifer().allDisposedObjects();
        assertTrue(allDisposedObjects.hasMoreElements());
        assertEquals(movieAdapter, allDisposedObjects.nextElement());
    }

    
    public void testClientSideActionWhereObjectChanged() throws Exception {
        system.setObjectPersistor(proxy);

        NakedObject directorAdapter = new DummyNakedObject(ResolveState.RESOLVED);

        NakedObject movieAdapter = new DummyNakedObject(ResolveState.RESOLVED);

        DummyObjectData movieData = new DummyObjectData(new DummyOid(12, true), Movie.class.getName(), true,
                 new DummyVersion(4));
         DummyValueData expectedMovieName = new DummyValueData("War of the Worlds", String.class.getName());
         DummyReferenceData expectedDirectorRef = new DummyReferenceData(new DummyOid(14, true), Person.class.getName(),
                 new DummyVersion(8));
         movieData.setFieldContent(new Data[] { expectedDirectorRef, expectedMovieName });

         DummyObjectData directorData = new DummyObjectData(new DummyOid(14, true), Person.class.getName(), true,
                 new DummyVersion(8));
         DummyValueData expectedDirectorName = new DummyValueData("Unknown", String.class.getName());
         directorData.setFieldContent(new Data[] { expectedDirectorName });

         expect(encoder.createGraphForChangedObject(movieAdapter, new KnownObjects())).andReturn(movieData);
         expect(encoder.createGraphForChangedObject(directorAdapter, new KnownObjects())).andReturn(directorData);

         
         ObjectData[] changes = new ObjectData[] { movieData, directorData };
         int[] types = new int[] { ClientTransactionEvent.CHANGE, ClientTransactionEvent.CHANGE };
         remoteInterface.executeClientAction(eq(session), aryEq(changes), aryEq(types));
         Version[] versionUpdates = new Version[] { new DummyVersion(5), new DummyVersion(9) };
         expectLastCall().andReturn(new DummyClientResultData(new ObjectData[] {movieData, directorData}, versionUpdates, new ObjectData[0]));
        
        
        replay();
        
        proxy.startTransaction();
        proxy.objectChanged(movieAdapter);
        proxy.objectChanged(directorAdapter);
        proxy.endTransaction();

        verify();
        
        assertEquals(new DummyVersion(5), movieAdapter.getVersion());
        assertEquals(new DummyVersion(9), directorAdapter.getVersion());
    }

    public void testClientSideActionWhereTransientObjectMadePersistent() throws Exception {
        system.setObjectPersistor(proxy);
        
        proxy.setOidGenerator(new SimpleOidGenerator(10));
        
        
        DummyObjectData movieData = new DummyObjectData(new DummyOid(-1), Movie.class.getName(), true, null);
        NullData directorData = new DummyNullValue(Person.class.getName());
        DummyValueData nameData = new DummyValueData("Star Wars", String.class.getName());
        movieData.setFieldContent(new Data[] { directorData, nameData });

        TestProxyOid newOid = new TestProxyOid(123, true);
        newOid.previous = new TestProxyOid(-1);

        DummyReferenceData updateData = new DummyReferenceData(newOid, "type", new DummyVersion(456));

        NakedObject transientObject  = new DummyNakedObject(newOid.previous, ResolveState.NEW);
        
        NakedObjectsContext.getObjectLoader().createAdapterForTransient(transientObject, false);
        
        // client needs to encode the object's transient aspects
        expect(encoder.createMakePersistentGraph(transientObject, new KnownObjects())).andReturn(movieData);

        // the server is called with data (movieData) for the object to be persisted
        remoteInterface.executeClientAction(eq(session), aryEq(new ReferenceData[] { movieData }), aryEq(new int[] { ClientTransactionEvent.ADD }));
        expectLastCall().andReturn(new DummyClientResultData(new ReferenceData[] { updateData }, null, new ObjectData[0]));

        replay();
        proxy.startTransaction();
        proxy.makePersistent(transientObject);
        proxy.endTransaction();
        verify();
    }

    public void testObjectChangedtOutsideTransaction() throws Exception {
        system.setObjectPersistor(proxy);

        DummyNakedObject transientObject = new DummyNakedObject();
        transientObject.setupResolveState(ResolveState.RESOLVED);
        try {
            proxy.objectChanged(transientObject);
            fail();
        } catch (TransactionException e) {}
    }

}

class TestCriteria implements CriteriaData  {
        private static final long serialVersionUID = 1L;

        public Class getCriteriaClass() {
            return null;
        }

        public boolean includeSubclasses() {
            return false;
        }

        public String getType() {
            return null;
        }
}
// Copyright (c) Naked Objects Group Ltd.
