package org.nakedobjects.nof.reflect.remote;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.reset;
import junit.framework.TestCase;

import org.easymock.EasyMock;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.nof.reflect.peer.ValuePeer;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nof.reflect.remote.data.DummyIdentityData;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.TestSystem;
import org.nakedobjects.testing.TestValueAdapter;

public class ProxyValueAssociationTest extends TestCase {
    private Distribution connection;
    private ObjectEncoder encoder;
    private DummyIdentityData expectedTarget;
    private NakedObject object;
    private ValuePeer peer;
    private ProxyValueAssociation proxy;
    private TestSystem system;

    private void replay() {
        EasyMock.replay(peer);
        EasyMock.replay(object);
        EasyMock.replay(encoder);
        EasyMock.replay(connection);
    }

    protected void setUp() throws Exception {
        system = new TestSystem();
        system.init();

        peer = (ValuePeer) createMock(ValuePeer.class);
        expect(peer.getIdentifier()).andReturn(new MemberIdentifierImpl("cls", "field"));

        connection = createMock(Distribution.class);

        encoder = createMock(ObjectEncoder.class);

        proxy = new ProxyValueAssociation(peer, connection, encoder);

        object = createMock(NakedObject.class);

        expectedTarget = new DummyIdentityData(new DummyOid(3), "type", null);
        expect(encoder.createIdentityData(object)).andReturn(expectedTarget);
        expect(object.getResolveState()).andReturn(ResolveState.RESOLVED);
    }
/*
    public void testPassesThroughConcurrencyExceptionWhenNoOidPresent() {
        distributionControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
        distributionControl.expectAndDefaultReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")), null);
        distributionControl.expectAndThrow(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")),
                new ConcurrencyException("", (Oid) null));

        replay();
        try {
            proxy.setValue(object, new StringAdapter("string"));
            fail();
        } catch (ConcurrencyException expected) {}
        verify();
    }

    public void testReloadsObjectAfterConcurrencyExceptionWhenOidPresent() {
        DummyOid oid = new DummyOid(5, true);
        system.addIdentityToObjectLoader(oid, new DummyNakedObject("object title"));
        
        distributionControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
        distributionControl.expectAndDefaultReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")), null);
        distributionControl.expectAndThrow(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")),
                new ConcurrencyException("concurrency failure...", oid));

        replay();
        try {
            proxy.setValue(object, new StringAdapter("string"));
            fail();
        } catch (ConcurrencyException expected) {
            assertEquals("Object automatically reloaded: object title\nconcurrency failure...", expected.getMessage());
        }
        verify();
    }
*/
    public void testSetValueCallPassedToDelegateWhenTargetIsTransient() {
        reset(encoder);
        reset(object);
        expect(object.getResolveState()).andReturn(ResolveState.TRANSIENT);

        peer.setValue(object, new TestValueAdapter("string"));

//        distributionControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
//        distributionControl.expectAndDefaultReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")), null);
        
      //  distributionControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
      
        
        replay();
        proxy.setValue(object, new TestValueAdapter("string"));
        verify();
    }
/*
    public void testSetValueCallPassedToDistributionWhenTargetIsPersistent() {
        distributionControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
        distributionControl.expectAndDefaultReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")), null);
        distributionControl.expectAndReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")),
                new ObjectData[0]);

        replay();
        proxy.setValue(object, new StringAdapter("string"));
        verify();
    }
/*
    public void testSetValueCausesUpdates() {
        ObjectData[] updates = new ObjectData[] { new DummyObjectData(new DummyOid(4), "field 1", false, null),
                new DummyObjectData(new DummyOid(7), "field 2", false, null) };
        distributionControl.setDefaultMatcher(MockControl.ARRAY_MATCHER);
        distributionControl.expectAndDefaultReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")), null);
        distributionControl.expectAndReturn(connection.setValue(NakedObjectsContext.getSession(), "field", expectedTarget, new DummyValueData("string")), updates);
        encoderControl.expectAndReturn(encoder.restore(updates[0]), null);
        encoderControl.expectAndReturn(encoder.restore(updates[1]), null);

        replay();
        proxy.setValue(object, new StringAdapter("string"));
        verify();
    }
*/
    private void verify() {
        EasyMock.verify(peer);
        EasyMock.verify(object);
        EasyMock.verify(encoder);
        EasyMock.verify(connection);
    }

}

// Copyright (c) Naked Objects Group Ltd.
