package test.org.nakedobjects.object.reflect;

import junit.framework.TestCase;

import org.nakedobjects.nof.core.util.NameConvertor;


public class NameConvertorTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(NameConvertorTest.class);
    }

    public void testNaturalNameAddsSpacesToCamelCaseWords() {
        assertEquals("Camel Case Word", NameConvertor.naturalName("CamelCaseWord"));
    }

    public void testNaturalNameAddsSpacesBeforeNumbers() {
        assertEquals("One 2 One", NameConvertor.naturalName("One2One"));
        assertEquals("Type 123", NameConvertor.naturalName("Type123"));
        assertEquals("4321 Go", NameConvertor.naturalName("4321Go"));
    }

    public void testNaturalNameRecognisesAcronymns() {
        assertEquals("TNT Power", NameConvertor.naturalName("TNTPower"));
        assertEquals("Spam RAM Can", NameConvertor.naturalName("SpamRAMCan"));
        assertEquals("DOB", NameConvertor.naturalName("DOB"));
    }

    public void testNaturalNameWithShortNames() {
        assertEquals("At", NameConvertor.naturalName("At"));
        assertEquals("I", NameConvertor.naturalName("I"));
    }

    public void testNaturalNameNoChange() {
        assertEquals("Camel Case Word", NameConvertor.naturalName("CamelCaseWord"));
        assertEquals("Almost Normal english sentence", NameConvertor.naturalName("Almost Normal english sentence"));
    }

    public void testPluralNameAdd_S() {
        assertEquals("Cans", NameConvertor.pluralName("Can"));
        assertEquals("Spaces", NameConvertor.pluralName("Space"));
        assertEquals("Noses", NameConvertor.pluralName("Nose"));
    }

    public void testPluralNameReplace_Y_With_IES() {
        assertEquals("Babies", NameConvertor.pluralName("Baby"));
        assertEquals("Cities", NameConvertor.pluralName("City"));
    }

    public void testPluralNameReplaceAdd_ES() {
        assertEquals("Foxes", NameConvertor.pluralName("Fox"));
        assertEquals("Bosses", NameConvertor.pluralName("Boss"));
    }

    public void testSimpleNameAllToLowerCase() {
        assertEquals("abcde", NameConvertor.simpleName("ABCDE"));
        assertEquals("camelcaseword", NameConvertor.simpleName("CamelCaseWord"));
    }

    public void testSimpleNameNoChanges() {
        assertEquals("nochanges", NameConvertor.simpleName("nochanges"));
    }

    public void testSimpleNameRemoveSpaces() {
        assertEquals("abcde", NameConvertor.simpleName("a bc  de "));
        assertEquals("twoparts", NameConvertor.simpleName("two parts"));
    }

}
// Copyright (c) Naked Objects Group Ltd.
