package test.org.nakedobjects.object.reflect;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.nof.core.adapter.PojoAdapter;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;



public class PojoAdapterTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(PojoAdapterTest.class);
    }

    private TestSystem system;

    protected void setUp() throws Exception {
        Logger.getRootLogger().setLevel(Level.OFF);
        system = new TestSystem();
        system.init();
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    private TestSpecification setupSpecificationTitleString(final String titleString) {
        TestSpecification spec = new TestSpecification("spec") {
            public String getFullName() {
                return TestPojo.class.getName();
            }
        };

        spec.setupTitle(titleString);
        system.addSpecificationToLoader(spec);
        return spec;
    }

    public void testEmptyTitle() {
        setupSpecificationTitleString("");
        NakedObject pa = new PojoAdapter(new TestPojo(), new DummyOid(1));
        assertEquals("", pa.titleString());
    }

    public void testTitleStringWhereSpecificationProvidesTitleFromObject() {
        setupSpecificationTitleString("object title from specification");

        PojoAdapter pa = new PojoAdapter(new TestPojo(), new DummyOid(1));
        pa.changeState(ResolveState.TRANSIENT);
        assertEquals("object title from specification", pa.titleString());
    }

    public void testTitleStringWhereSpecificationReturnNullAsTitle() {
        setupSpecificationTitleString(null);

        PojoAdapter pa = new PojoAdapter(new TestPojo(), new DummyOid(1));
        pa.changeState(ResolveState.TRANSIENT);
        assertEquals("A spec (singular)", pa.titleString());
    }
}
// Copyright (c) Naked Objects Group Ltd.
