package test.org.nakedobjects.object.reflect.defaults;

import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.persist.ObjectPersistenceException;
import org.nakedobjects.noa.reflect.OneToManyAssociation;
import org.nakedobjects.nof.core.context.StaticContext;
import org.nakedobjects.nof.reflect.spec.OneToManyAssociationImpl;
import org.nakedobjects.testing.DummyNakedCollection;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.TestPersistor;
import org.nakedobjects.testing.TestSpecification;

import test.org.nakedobjects.object.NakedObjectTestCase;
import test.org.nakedobjects.object.reflect.DummyOneToManyPeer;


public class OneToManyAssociationTest extends NakedObjectTestCase {
    // private static final String FIELD_LABEL = "Members";
    private static final String FIELD_ID = "members";
    private static final String FIELD_NAME = "Members";
    private NakedObject nakedObject;
    private NakedObject associate;
    private OneToManyAssociation association;
    private TestSpecification type;
    private DummyOneToManyPeer associationDelegate;

    public OneToManyAssociationTest(final String name) {
        super(name);
    }

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(OneToManyAssociationTest.class));
    }

    public void setUp() throws ObjectPersistenceException {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        StaticContext.createInstance().setObjectPersistor(new TestPersistor());

        nakedObject = new DummyNakedObject();
        ((DummyNakedObject) nakedObject).setupResolveState(ResolveState.RESOLVED);
        associate = new DummyNakedObject();
        ((DummyNakedObject) associate).setupResolveState(ResolveState.RESOLVED);

        type = new TestSpecification();
        associationDelegate = new DummyOneToManyPeer();
        association = new OneToManyAssociationImpl(FIELD_NAME, type, associationDelegate);
    }

    public void testType() {
        assertEquals(type, association.getSpecification());
    }

    public void testSet() {
        association.addElement(nakedObject, associate);
        associationDelegate.assertAction(0, "add " + nakedObject);
        associationDelegate.assertAction(1, "add " + associate);
    }

    public void testClear() {
        association.removeElement(nakedObject, associate);
        associationDelegate.assertAction(0, "remove " + nakedObject);
        associationDelegate.assertAction(1, "remove " + associate);
    }

    public void testClearWithNull() {
        try {
            association.removeElement(nakedObject, null);
            fail();
        } catch (IllegalArgumentException expected) {}
        associationDelegate.assertActions(0);
    }

    public void testSetWithNull() {
        try {
            association.addElement(nakedObject, null);
            fail();
        } catch (IllegalArgumentException expected) {}
        associationDelegate.assertActions(0);
    }

    public void testGet() {
        NakedCollection collection = new DummyNakedCollection();
        associationDelegate.getCollection = collection;
        Naked returnedObject = association.get(nakedObject);
        assertEquals(collection, returnedObject);
    }

    public void testName() {
        assertEquals(FIELD_NAME, association.getId());
    }

    public void testLabel() {
        assertEquals(FIELD_NAME, association.getName());
    }
}
// Copyright (c) Naked Objects Group Ltd.
