package test.org.nakedobjects.object.reflect.defaults;

import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.reflect.OneToOneAssociation;
import org.nakedobjects.nof.reflect.spec.OneToOneAssociationImpl;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;

import test.org.nakedobjects.object.NakedObjectTestCase;
import test.org.nakedobjects.object.reflect.DummyOneToOnePeer;


public class OneToOneAssociationTest extends NakedObjectTestCase {
    private static final String NAME = "SmallPerson";
    private static final String FIELD_NAME = "SmallPerson";
    private static final String FIELD_LABEL = "Small Person";
    private NakedObject nakedObject;
    private OneToOneAssociation association;
    private NakedObject associate;
    private TestSpecification type;
    private DummyOneToOnePeer associationDelegate;
    private TestSystem system;

    public OneToOneAssociationTest(final String name) {
        super(name);
    }

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(OneToOneAssociationTest.class));
    }

    protected void setUp() throws Exception {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        system = new TestSystem();
        system.init();

        nakedObject = new DummyNakedObject();
        ((DummyNakedObject) nakedObject).setupResolveState(ResolveState.RESOLVED);
        associate = new DummyNakedObject();
        ((DummyNakedObject) associate).setupResolveState(ResolveState.RESOLVED);

        associationDelegate = new DummyOneToOnePeer();
        type = new TestSpecification();
        association = new OneToOneAssociationImpl(NAME, type, associationDelegate);
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testType() {
        assertEquals(type, association.getSpecification());
    }

    public void testSet() {
        association.setAssociation(nakedObject, associate);
        associationDelegate.assertAction(0, "associate " + nakedObject);
        associationDelegate.assertAction(1, "associate " + associate);
    }

    public void testRemove() {
        association.clearAssociation(nakedObject, associate);
        associationDelegate.assertAction(0, "clear " + nakedObject);
        associationDelegate.assertAction(1, "clear " + associate);
    }

    public void testGet() {
        Naked object = association.get(nakedObject);
        assertNull(object);

        associationDelegate.getObject = new DummyNakedObject();
        object = association.get(nakedObject);
        assertEquals(associationDelegate.getObject, object);
        associationDelegate.assertAction(0, "get " + nakedObject);
    }

    public void testName() {
        assertEquals(FIELD_NAME, association.getId());
    }

    public void testLabel() {
        assertEquals(FIELD_LABEL, association.getName());
    }

    public void testIsEmpty() {
        assertFalse(association.isEmpty(nakedObject));
        associationDelegate.assertAction(0, "empty " + nakedObject);

        associationDelegate.isEmpty = true;
        assertTrue(association.isEmpty(nakedObject));
        associationDelegate.assertAction(1, "empty " + nakedObject);
    }
}
// Copyright (c) Naked Objects Group Ltd.
